@@ -38,11 +38,12 @@
 #include <QGridLayout>
 #include <QLabel>
 #include <QLoggingCategory>
+#include <QMimeDatabase>
 #include <QUrl>
 #include <QCheckBox>
 #include <QComboBox>
 #include <QDBusObjectPath>
-#include <libfm-qt/filedialog.h>
+#include <libfm-qt6/filedialog.h>
 
 // Keep in sync with qflatpakfiledialog from flatpak-platform-plugin
 Q_DECLARE_METATYPE(LXQt::FileChooserPortal::Filter)
@@ -163,6 +164,22 @@ namespace LXQt
     {
     }
 
+    // The portal may send us null terminated strings. Make sure to strip the extranous \0
+    // in favor of the implicit \0.
+    // QByteArrays are implicitly terminated already.
+    static QUrl decodeFileName(const QByteArray &name)
+    {
+        QByteArray decodedName = name;
+        while (decodedName.endsWith('\0')) {
+            decodedName.chop(1);
+        }
+        QString str = QFile::decodeName(decodedName);
+        if (!str.isEmpty()) {
+            return QUrl::fromLocalFile(str);
+        }
+        return QUrl();
+    }
+
     uint FileChooserPortal::OpenFile(const QDBusObjectPath &handle,
             const QString &app_id,
             const QString &parent_window,
@@ -181,9 +198,9 @@ namespace LXQt
         bool directory = false;
         bool modalDialog = true;
         bool multipleFiles = false;
+        QUrl currentFolder;
         QStringList nameFilters;
-        QStringList mimeTypeFilters;
-        QString selectedMimeTypeFilter;
+        QString selectedNameFilter;
         // mapping between filter strings and actual filters
         QMap<QString, FilterList> allFilters;
 
@@ -201,10 +218,14 @@ namespace LXQt
             directory = options.value(QStringLiteral("directory")).toBool();
         }
 
-        ExtractFilters(options, nameFilters, mimeTypeFilters, allFilters, selectedMimeTypeFilter);
+        if (options.contains(QStringLiteral("current_folder"))) {
+            currentFolder = decodeFileName(options.value(QStringLiteral("current_folder")).toByteArray());
+        }
+
+        ExtractFilters(options, nameFilters, allFilters, selectedNameFilter);
 
         // for handling of options - choices
-        QScopedPointer<QWidget> optionsWidget;
+        std::unique_ptr<QWidget> optionsWidget;
         // to store IDs for choices along with corresponding comboboxes/checkboxes
         QMap<QString, QCheckBox *> checkboxes;
         QMap<QString, QComboBox *> comboboxes;
@@ -222,22 +243,22 @@ namespace LXQt
         if (!acceptLabel.isEmpty())
             fileDialog->setLabelText(QFileDialog::Accept, acceptLabel);
 
-        if (mLastVisitedDirs.count(parent_window) > 0) {
+        if (currentFolder.isValid()) {
+            fileDialog->setDirectory(currentFolder);
+        } else if (mLastVisitedDirs.count(parent_window) > 0) {
             fileDialog->setDirectory(mLastVisitedDirs[parent_window]);
         }
 
-        bool bMimeFilters = false;
-        if (!mimeTypeFilters.isEmpty()) {
-            fileDialog->setMimeTypeFilters(mimeTypeFilters);
-            fileDialog->selectMimeTypeFilter(selectedMimeTypeFilter);
-            bMimeFilters = true;
-        } else {
+        if (!nameFilters.isEmpty()) {
             fileDialog->setNameFilters(nameFilters);
+            fileDialog->selectNameFilter(selectedNameFilter);
         }
 
+        bool bHasOptions = false;
         if (optionsWidget) {
             if (auto layout = fileDialog->dialog().layout()) {
-                layout->addWidget(optionsWidget.get());
+                layout->addWidget(optionsWidget.release());
+                bHasOptions = true;
             }
         }
 
@@ -255,18 +276,13 @@ namespace LXQt
             results.insert(QStringLiteral("uris"), files);
             results.insert(QStringLiteral("writable"), true);
 
-            if (optionsWidget) {
+            if (bHasOptions) {
                 QVariant choices = EvaluateSelectedChoices(checkboxes, comboboxes);
                 results.insert(QStringLiteral("choices"), choices);
             }
 
             // try to map current filter back to one of the predefined ones
-            QString selectedFilter;
-            if (bMimeFilters) {
-                selectedFilter = fileDialog->selectedMimeTypeFilter();
-            } else {
-                selectedFilter = fileDialog->selectedNameFilter();
-            }
+            QString selectedFilter = fileDialog->selectedNameFilter();
             if (allFilters.contains(selectedFilter)) {
                 results.insert(QStringLiteral("current_filter"), QVariant::fromValue<FilterList>(allFilters.value(selectedFilter)));
             }
@@ -299,8 +315,7 @@ namespace LXQt
         QUrl currentFolder;
         QUrl currentFile;
         QStringList nameFilters;
-        QStringList mimeTypeFilters;
-        QString selectedMimeTypeFilter;
+        QString selectedNameFilter;
         // mapping between filter strings and actual filters
         QMap<QString, FilterList> allFilters;
 
@@ -315,17 +330,17 @@ namespace LXQt
         }
 
         if (options.contains(QStringLiteral("current_folder"))) {
-            currentFolder = QUrl::fromLocalFile(options.value(QStringLiteral("current_folder")).toString());
+            currentFolder = decodeFileName(options.value(QStringLiteral("current_folder")).toByteArray());
         }
 
         if (options.contains(QStringLiteral("current_file"))) {
-            currentFile = QUrl::fromLocalFile(options.value(QStringLiteral("current_file")).toString());
+            currentFile = decodeFileName(options.value(QStringLiteral("current_file")).toByteArray());
         }
 
-        ExtractFilters(options, nameFilters, mimeTypeFilters, allFilters, selectedMimeTypeFilter);
+        ExtractFilters(options, nameFilters, allFilters, selectedNameFilter);
 
         // for handling of options - choices
-        QScopedPointer<QWidget> optionsWidget;
+        std::unique_ptr<QWidget> optionsWidget;
         // to store IDs for choices along with corresponding comboboxes/checkboxes
         QMap<QString, QCheckBox *> checkboxes;
         QMap<QString, QComboBox *> comboboxes;
@@ -367,18 +382,16 @@ namespace LXQt
             fileDialog->selectFile(dir.resolved(relative_file));
         }
 
-        bool bMimeFilters = false;
-        if (!mimeTypeFilters.isEmpty()) {
-            fileDialog->setMimeTypeFilters(mimeTypeFilters);
-            fileDialog->selectMimeTypeFilter(selectedMimeTypeFilter);
-            bMimeFilters = true;
-        } else {
+        if (!nameFilters.isEmpty()) {
             fileDialog->setNameFilters(nameFilters);
+            fileDialog->selectNameFilter(selectedNameFilter);
         }
 
+        bool bHasOptions = false;
         if (optionsWidget) {
             if (auto layout = fileDialog->dialog().layout()) {
-                layout->addWidget(optionsWidget.get());
+                layout->addWidget(optionsWidget.release());
+                bHasOptions = true;
             }
         }
 
@@ -390,18 +403,13 @@ namespace LXQt
             }
             results.insert(QStringLiteral("uris"), files);
 
-            if (optionsWidget) {
+            if (bHasOptions) {
                 QVariant choices = EvaluateSelectedChoices(checkboxes, comboboxes);
                 results.insert(QStringLiteral("choices"), choices);
             }
 
             // try to map current filter back to one of the predefined ones
-            QString selectedFilter;
-            if (bMimeFilters) {
-                selectedFilter = fileDialog->selectedMimeTypeFilter();
-            } else {
-                selectedFilter = fileDialog->selectedNameFilter();
-            }
+            QString selectedFilter = fileDialog->selectedNameFilter();
             if (allFilters.contains(selectedFilter)) {
                 results.insert(QStringLiteral("current_filter"), QVariant::fromValue<FilterList>(allFilters.value(selectedFilter)));
             }
@@ -499,9 +507,8 @@ namespace LXQt
 
     void FileChooserPortal::ExtractFilters(const QVariantMap &options,
             QStringList &nameFilters,
-            QStringList &mimeTypeFilters,
             QMap<QString, FilterList> &allFilters,
-            QString &selectedMimeTypeFilter)
+            QString &selectedNameFilter)
     {
         if (options.contains(QStringLiteral("filters"))) {
             const FilterListList filterListList = qdbus_cast<FilterListList>(options.value(QStringLiteral("filters")));
@@ -511,17 +518,15 @@ namespace LXQt
                     if (filterStruct.type == 0) {
                         filterStrings << filterStruct.filterString;
                     } else {
-                        mimeTypeFilters << filterStruct.filterString;
-                        allFilters[filterStruct.filterString] = filterList;
+                        filterStrings << NameFiltersForMimeType(filterStruct.filterString);
                     }
                 }
 
                 if (!filterStrings.isEmpty()) {
-                    QString userVisibleName = filterList.userVisibleName;
                     const QString filterString = filterStrings.join(QLatin1Char(' '));
-                    const QString nameFilter = QStringLiteral("%2 (%1)").arg(filterString, userVisibleName);
+                    const QString nameFilter = QStringLiteral("%2 (%1)").arg(filterString, filterList.userVisibleName);
                     nameFilters << nameFilter;
-                    allFilters[filterList.userVisibleName] = filterList;
+                    allFilters[nameFilter] = filterList;
                 }
             }
         }
@@ -529,20 +534,40 @@ namespace LXQt
         if (options.contains(QStringLiteral("current_filter"))) {
             FilterList filterList = qdbus_cast<FilterList>(options.value(QStringLiteral("current_filter")));
             if (filterList.filters.size() == 1) {
+                QStringList filterStrings;
                 Filter filterStruct = filterList.filters.at(0);
                 if (filterStruct.type == 0) {
+                    filterStrings << filterStruct.filterString;
+                } else {
+                    filterStrings << NameFiltersForMimeType(filterStruct.filterString);
+                }
+
+                if (!filterStrings.isEmpty()) {
                     // make the relevant entry the first one in the list of filters,
                     // since that is the one that gets preselected by KFileWidget::setFilter
-                    QString userVisibleName = filterList.userVisibleName;
-                    QString nameFilter = QStringLiteral("%2 (%1)").arg(filterStruct.filterString, userVisibleName);
+                    const QString filterString = filterStrings.join(QLatin1Char(' '));
+                    const QString nameFilter = QStringLiteral("%2 (%1)").arg(filterString, filterList.userVisibleName);
                     nameFilters.removeAll(nameFilter);
                     nameFilters.push_front(nameFilter);
-                } else {
-                    selectedMimeTypeFilter = filterStruct.filterString;
+                    selectedNameFilter = nameFilter;
                 }
             } else {
                 qCDebug(XdgDesktopPortalLxqtFileChooser) << "Ignoring 'current_filter' parameter with 0 or multiple filters specified.";
             }
         }
     }
+
+    QStringList FileChooserPortal::NameFiltersForMimeType(const QString &mimeType)
+    {
+        QMimeDatabase db;
+        QMimeType mime(db.mimeTypeForName(mimeType));
+
+        if (mime.isValid()) {
+            if (mime.isDefault()) {
+                return QStringList("*");
+            }
+            return mime.globPatterns();
+        }
+        return QStringList();
+    }
 }
